@@ -78,11 +78,16 @@ custom_shell_surface_on_window_map (GtkWidget *widget, CustomShellSurface *self)
     // attached, so we don't need to commit. If this is removed, test-window-with-initially-attached-buffer should fail.
     wl_surface_attach (wl_surface, NULL, 0, 0);
 
+    self->configured = FALSE;
     self->virtual->map (self, wl_surface);
     gdk_window_set_priv_mapped (gdk_window);
 
     wl_surface_commit (wl_surface);
-    wl_display_roundtrip (gdk_wayland_display_get_wl_display (gdk_display_get_default ()));
+
+    struct wl_display *display = gdk_wayland_display_get_wl_display (gdk_display_get_default ());
+    while (!self->configured) {
+        wl_display_roundtrip (display);
+    }
 }
 
 void
