@@ -110,6 +110,7 @@ layer_surface_handle_configure (void *data,
     LayerSurface *self = data;
 
     zwlr_layer_surface_v1_ack_configure (surface, serial);
+    self->super.configured = TRUE;
 
     self->last_configure_size = (GtkRequisition) {
         .width = (gint)w,
@@ -210,6 +211,10 @@ layer_surface_finalize (CustomShellSurface *super)
     LayerSurface *self = (LayerSurface *)super;
     custom_shell_surface_unmap (super);
     g_free ((gpointer)self->name_space);
+    GdkDisplay *gdk_display = gdk_display_get_default ();
+    // Disconnect the monitor change signals
+    g_signal_handlers_disconnect_by_data (gdk_display, self);
+    g_clear_object (&self->monitor);
 }
 
 static struct xdg_popup *
@@ -298,6 +303,15 @@ layer_surface_on_size_allocate (GtkWidget *_gtk_window,
     }
 }
 
+static void monitor_changed(GdkDisplay* self, GdkMonitor* monitor, LayerSurface *layer_surface) {
+    (void)self; (void)monitor;
+    // If the surface has a monitor set, it's in charge of responding to monitor changes
+    // Don't remap unless the surface is currently mapped (has a layer surface)
+    if (layer_surface->monitor == NULL && layer_surface->layer_surface) {
+        custom_shell_surface_remap ((CustomShellSurface *)layer_surface);
+    }
+}
+
 LayerSurface *
 layer_surface_new (GtkWindow *gtk_window)
 {
@@ -323,6 +337,9 @@ layer_surface_new (GtkWindow *gtk_window)
 
     gtk_window_set_decorated (gtk_window, FALSE);
     g_signal_connect (gtk_window, "size-allocate", G_CALLBACK (layer_surface_on_size_allocate), self);
+    GdkDisplay *gdk_display = gdk_display_get_default ();
+    g_signal_connect (gdk_display, "monitor-added", G_CALLBACK (monitor_changed), self);
+    g_signal_connect (gdk_display, "monitor-removed", G_CALLBACK (monitor_changed), self);
 
     return self;
 }
@@ -341,7 +358,10 @@ layer_surface_set_monitor (LayerSurface *self, GdkMonitor *monitor)
 {
     if (monitor) g_return_if_fail (GDK_IS_WAYLAND_MONITOR (monitor));
     if (monitor != self->monitor) {
-        self->monitor = monitor;
+        g_clear_object (&self->monitor);
+        if (monitor) {
+            self->monitor = g_object_ref (monitor);
+        }
         if (self->layer_surface) {
             custom_shell_surface_remap ((CustomShellSurface *)self);
         }
